;; Generated package description from adjust-parens.el
(define-package "adjust-parens" "1.1" "Indent and dedent Lisp code, automatically adjust close parens" 'nil :url "http://elpa.gnu.org/packages/adjust-parens.html")
